@@ -7,4 +7,4 @@ globalVariables(c("zone", "region", "parent_class", "child_class",
   "interval_id", "interval_to_id", "db", "period_id", "time_from",
   "time_to", "Version", "rplexos", "d", "error", "interval",
   "tables", "properties", "i", "phase", "timestep", "name", "type",
-  "folder", "class_group", "element", "parent"))
+  "folder", "class_group", "element", "parent", "time", "start", "end"))
